##############################################################################
#
# Copyright (c) 2010 ViFiB SARL and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zc.buildout.download import Download, ChecksumError, check_md5sum, \
                                 locate_at, url_opener

import os
import urllib
import re
import shutil
import urlparse
import tempfile
import logging
import zc.buildout

def network_cache__init__(self, options={}, cache=-1, namespace=None,
                   offline=-1, fallback=False, hash_name=False, logger=None):
    self.network_cache = options.get('network-cache', None)
    self._original__init__(options=options, cache=cache, namespace=namespace,
                   offline=offline, fallback=fallback, hash_name=hash_name,
                   logger=logger)

def download_network_cached(self, path, md5sum):
    """Download from a network cache provider

    If something fail (providor be offline, or md5sum fail), we ignore
    network cached files.

    return True if download succeeded.
    """
    url = os.path.join(self.network_cache, md5sum)
    self.logger.info('Downloading from network cache %s' % url)
    try:
        path, headers = urllib.urlretrieve(url, path)
        if not check_md5sum(path, md5sum):
            self.logger.info('MD5 checksum mismatch downloading %r' % url)
            return False
    except IOError, e:
        self.logger.info('Fail to download from network cache %s' % url)
        return False

    return True

def upload_network_cached(self, path, md5sum):
    """Upload file to a network cache server"""
    try:
        f = open(path, 'r')
        data = f.read()
        url = os.path.join(self.network_cache, md5sum)
        try:
            result = urllib.urlopen(url, urllib.urlencode({
                "data" : data}))
        except (IOError,EOFError), e:
            self.logger.info('Fail to upload cache on %s' % url)
    finally:
        f.close()
    return True

# The follow method is original Download.download with few changes.
def download_with_network_cache(self, url, md5sum=None, path=None):
    """Download a file from a URL to a given or temporary path.

    An online resource is always downloaded to a temporary file and moved
    to the specified path only after the download is complete and the
    checksum (if given) matches. If path is None, the temporary file is
    returned and the client code is responsible for cleaning it up.

    """
    # Make sure the drive letter in windows-style file paths isn't
    # interpreted as a URL scheme.
    if re.match(r"^[A-Za-z]:\\", url):
        url = 'file:' + url

    parsed_url = urlparse.urlparse(url, 'file')
    url_scheme, _, url_path = parsed_url[:3]
    if url_scheme == 'file':
        self.logger.debug('Using local resource %s' % url)
        if not check_md5sum(url_path, md5sum):
            raise ChecksumError(
                'MD5 checksum mismatch for local resource at %r.' %
                url_path)
        return locate_at(url_path, path), False

    if self.offline:
        raise zc.buildout.UserError(
            "Couldn't download %r in offline mode." % url)

    self.logger.info('Downloading %s' % url)
    urllib._urlopener = url_opener
    handle, tmp_path = tempfile.mkstemp(prefix='buildout-')
    try:
        try:
            is_downloaded = False
            if None not in [md5sum , self.network_cache]:
                is_downloaded = self.download_network_cached(tmp_path, md5sum)

            if not is_downloaded:
                tmp_path, headers = urllib.urlretrieve(url, tmp_path)
                if not check_md5sum(tmp_path, md5sum):
                    raise ChecksumError(
                        'MD5 checksum mismatch downloading %r' % url)
                self.upload_network_cached(tmp_path, md5sum)
        finally:
            os.close(handle)
    except:
        os.remove(tmp_path)
        raise

    if path:
        shutil.move(tmp_path, path)
        return path, False
    else:
        return tmp_path, True

def install(buildout=None):
    logger = logging.getLogger(__name__)
    logger.info("Extending Buildout Download API to support network cache")
    Download._original__init__ = Download.__init__
    Download.__init__ = network_cache__init__
    Download.upload_network_cached = upload_network_cached
    Download.download_network_cached = download_network_cached
    Download.download = download_with_network_cache
